// *******************************************************************
//    IBM Rational Ltd  Rhapsody in C++ Simplified eXecution Framework
//   ================== ==============================================
//
//    Model Element   : OMTimeout
//    File Path       : OMTimeout.cpp 
//    Licensed Materials - Property of IBM
//    (c) Copyright IBM Corporation 2008, 2016. All Rights Reserved.
//    
//    Note to U.S. Government Users Restricted Rights:  
//    Use, duplication or disclosure restricted by GSA ADP Schedule 
//    Contract with IBM Corp.
// *******************************************************************


#include "OMTimeout.h"
#include "OMMainThread.h"
#include "OMTimeoutPool.h"
#include "oxf.h"
//## package SXF::OMGeneric_Pkg

//## class OMTimeout
//#[ requirement(s)
// Realizes requirement Timeout Class #SXF_LR.1.7
//#]
//#[ requirement(s)
// Realizes requirement Timeout Constructor #SXF_LR.1.7.2:
// OMTimeout constructor shall create timeout instance and initialize its attributes:
//  - lId to OMEvent::sxfTimeoutEventId
//  - deliveryTime to 0U
//  - itsOMTimeoutPool to NULL
//  - destination to NULL
//  - itsNextTimeout to NULL.
//  - deleteAfterConsume to true.
// The constructor shall be public.
//#]
OMTimeout::OMTimeout(void) : OMEvent(getSxfTimeoutEventId()),deliveryTime(0U), itsNextTimeout(NULL), itsOMTimeoutPool(NULL) {
    //#[ operation OMTimeout()
    //#]
}

//#[ requirement(s)
// Realizes requirement Return Timeout to Free List #SXF_LR.1.7.9:
// This function shall put timeout to list of free elements via OMTimeoutPool::putBack() function call with 'this' argument.
// The function shall be public.
//#]
void OMTimeout::ReturnTmToPool(void) {
    //#[ operation ReturnTmToPool()
    /*LDRA_INSPECTED 72 D : MISRA-C++:2008 5-0-1: use "this" as an argument to modify link lists of free and busy timeouts*/
    itsOMTimeoutPool->putBack(this);
    
    //#]
}

//#[ requirement(s)
// Realizes requirement Set Head ofTimeout List #SXF_LR.1.7.4:
// This function shall set a head of timeout list.
// It has one parameter of type 'OMTimeout* const'.
// The function shall be public.
//#]
/*LDRA_INSPECTED 62 D 1 : MISRA-C++:2008 7-1-2: "aTimeout" parameter cannot be const */
void OMTimeout::setItsNextTimeout(OMTimeout* const aTimeout) {
    //#[ operation setItsNextTimeout(OMTimeout)
    itsNextTimeout = aTimeout;
    //#]
}

//#[ requirement(s)
// Realizes requirement Timeout is Expired #SXF_LR.1.7.3:
// This function shall check if timeout is expired.
// It has two parameters:
// - currTime of OMOSSpecific::timeUnit type,
// - lastTime of OMOSSpecific::timeUnit type.
// If lastTime is less of equal currTime, the function returns true if its deliveryTime value is less or equal to currTime.
// Otherwise the function returns true if lastTime less or equal deliveryTime.
// The function shall be public and const.
//#]
bool OMTimeout::isExpired(const OMOSSpecific::timeUnit currTime, const OMOSSpecific::timeUnit lastTime) const {
    //#[ operation isExpired(timeUnit,timeUnit) const
    bool b;
    if (lastTime <= currTime)
    {
    	// current time is bigger then previous one
    	// it means time counter didn't reset 
    	b = (deliveryTime <= currTime);
    } 
    else
    {  
       b = (lastTime <= deliveryTime);
    }
    return b;
    //#]
}

//#[ requirement(s)
// Realizes requirement Cancel Timeout #SXF_LR.1.7.6:
// This function shall remove timeout from list of busy elements and move it to list of free elements. 
// Function sets the destination of the OMTimeout to NULL.
// If itsOMTimeoutPool is not NULL the function calls removeTm() on the itsOMTimeoutPool using the this-pointer as call-parameter.
// If itsOMTimeoutPool is NULL the function makes nothing.
// It always returns bool true.
// The function shall be public.
//#]
bool OMTimeout::cancel(void) {
    //#[ operation cancel()
    // remove timeout from itsBusyList
    /*LDRA_INSPECTED 72 D : MISRA-C++:2008 5-0-1: use "this" as an argument to modify link lists of free and busy timeouts*/
    if (itsOMTimeoutPool != NULL)
    {
    	itsOMTimeoutPool->removeTm( this );
    }
    return true;
    
    //#]
}

//#[ requirement(s)
// Realizes requirement Set Timeout Expiration Time #SXF_LR.1.7.7:
// This function shall assign the value of  'p_deliveryTime' parameter to OMTimeout::deliveryTime attribute.
//#]
void OMTimeout::setDeliveryTime(const OMOSSpecific::timeUnit p_deliveryTime) {
    //#[ operation setDeliveryTime(timeUnit)
    deliveryTime = p_deliveryTime;
    //#]
}

//#[ requirement(s)
// Realizes requirement Initialize Timeout #SXF_LR.1.7.8:
// OMTimeout::initTimeout function has two parameters:
//  -  'nextTm' is of type 'OMTimeout* const'.
//  -  'tmPoolAddress' is of type 'OMTimeoutPool* const'.
// The function shall set itsNextTimeout to the value of its first parameter 'nextTm' and
// set itsOMTimeoutPool to the value of its second parameter 'tmPoolAddress'.
// The function shall be public.
//#]
/*LDRA_INSPECTED 62 D 1 : MISRA-C++:2008 7-1-2: "nextTm" parameter cannot be const */
/*LDRA_INSPECTED 62 D 2 : MISRA-C++:2008 7-1-2: "tmPoolAddress" parameter cannot be const */
void OMTimeout::initTimeout(OMTimeout* const nextTm, OMTimeoutPool* const tmPoolAddress) {
    //#[ operation initTimeout(OMTimeout,OMTimeoutPool)
    itsNextTimeout = nextTm;
    itsOMTimeoutPool = tmPoolAddress;
    
    //#]
}

//#[ requirement(s)
// Realizes requirement Get Its Next Timeout #SXF_LR.1.7.11:
//  itsNextTimeout attribute shall point to the next timeout in the busy or free list of timeouts.
// The getItsNextTimeout function shall return its value.
// The return value type is 'OMTimeout*'.
// The function shall be public.
//#]
OMTimeout* OMTimeout::getItsNextTimeout(void) const {
    //#[ operation getItsNextTimeout() const
    return itsNextTimeout;
    //#]
}

//#[ requirement(s)
// Realizes requirement Get DeliveryTime #SXF_LR.1.7.1:
// The deliveryTime attribute contains timeout expiration time.
// Its type is  'OMOSSpecific::timeUnit'.
// OMTimeout::getDeliveryTime function shall return its value.
// The function shall be public and const.
//#]
OMOSSpecific::timeUnit OMTimeout::getDeliveryTime(void) const {
    //#[ operation getDeliveryTime() const
    return deliveryTime;
    //#]
}

/*********************************************************************
	File Path	: ../OMTimeout.cpp
*********************************************************************/
